#include "stdafx.h"
#include <npapi.h>
#include <npfunctions.h>

NPNetscapeFuncs npn;

class Scriptable : public NPObject
{
public:
    static NPClass npClass;

    Scriptable() = default;

    void SetNPP(NPP npp)
    {
        this->npp = npp;
    }

private:
    static NPObject* Create(NPP npp, NPClass *aClass)
    {
        return static_cast<NPObject*>(new Scriptable());
    }

    static void Destroy(NPObject *npobj)
    {
        delete static_cast<Scriptable*>(npobj);
    }

    static bool HasMethod(NPObject *npobj, NPIdentifier name)
    {
        if(npn.getstringidentifier("test") == name)
        {
            return true;
        }

        return false;
    }

    static bool Invoke(NPObject *npobj, NPIdentifier name, const NPVariant *args, uint32_t argCount, NPVariant *result)
    {
        if(npn.getstringidentifier("test") == name)
        {
            auto inst = static_cast<Scriptable*>(npobj);

            NPObject* obj;
            npn.getvalue(inst->npp, NPNVPluginElementNPObject, &obj);

            NPVariant result;
            npn.invoke(inst->npp, obj, npn.getstringidentifier("onevent"), nullptr, 0, &result);

            npn.releasevariantvalue(&result);
            npn.releaseobject(obj);

            return true;
        }

        return false;
    }

    NPP npp;
};

NPClass Scriptable::npClass = {
    NP_CLASS_STRUCT_VERSION,
    Create,
    Destroy,
    nullptr,
    HasMethod,
    Invoke,
};


class Plugin
{
public:
    Plugin() = default;
    ~Plugin() = default;

    NPP instance = nullptr;
};

NPError NPP_New(NPMIMEType pluginType, NPP instance, uint16_t mode, int16_t argc, char *argn[], char *argv[], NPSavedData *saved)
{
    auto inst = new Plugin();
    instance->pdata = inst;
    inst->instance = instance;
    return NPERR_NO_ERROR;
}

NPError NPP_Destroy(NPP instance, NPSavedData **save)
{
    delete reinterpret_cast<Plugin*>(instance->pdata);
    return NPERR_NO_ERROR;
}

NPError NPP_GetValue(NPP instance, NPPVariable variable, void *ret_value)
{
    if(variable == NPPVpluginScriptableNPObject)
    {
        auto obj = npn.createobject(instance, &Scriptable::npClass);
        reinterpret_cast<Scriptable*>(obj)->SetNPP(instance);
        *((NPObject**)ret_value) = obj;
        return NPERR_NO_ERROR;
    }

    return NPERR_GENERIC_ERROR;
}

NPError WINAPI NP_Initialize(NPNetscapeFuncs* browserFuncs)
{
    memcpy(&::npn, browserFuncs, sizeof(browserFuncs->size) + browserFuncs->size);
    return NPERR_NO_ERROR;
}

NPError WINAPI NP_GetEntryPoints(NPPluginFuncs* pluginFuncs)
{
    pluginFuncs->newp = NPP_New;
    pluginFuncs->destroy = NPP_Destroy;
    pluginFuncs->getvalue = NPP_GetValue;
    //pluginFuncs->event = NPP_HandleEvent;
    return NPERR_NO_ERROR;
}

NPError WINAPI NP_Shutdown()
{
    return NPERR_NO_ERROR;
}
